local json = require("json")
local utils = require("utils")
local const = require("const")
local pl_pretty = require("pl.pretty")
local pl_string = require("pl.stringx")
local pl_dir = require("pl.dir")
local pl_path = require("pl.path")
local pl_file = require("pl.file")
local resty_string = require("resty.string")


local function check_config_params(log_level, log_max_size, size_unit, log_root_path)

    if ((not log_level) or (not utils.is_in_array({"DEBUG", "INFO", "WARN", "ERROR", "FATAL"}, log_level))) then
        return false
    end

    if ((not size_unit) or (not utils.is_in_array({"MB", "GB"}, size_unit))) then
        return false
    end

    if (not pl_path.isabs(log_root_path)) then
        return false
    end
    
    return true
end


local function set_log_config(log_level, log_max_size, size_unit, log_root_path)
    if (size_unit == "GB") then 
        log_max_size = log_max_size * 1024 -- to MB unit
    end

    if ("WARN" == log_level) then
        log_level = "WARNING"
    end

    local ok1, msg1 = utils.set_data(const.data_key.logger.dir, log_root_path)
    local ok2, msg2 = utils.set_data(const.data_key.logger.max_size, tostring(log_max_size))
    local ok3, msg3 = utils.set_data(const.data_key.logger.level, log_level)
    if (ok1 and ok2 and ok3) then
        return true
    end

    utils.log_error("set log config failed" .. msg1 .. msg2 .. msg3)
    utils.error_exit(500, "set log config failed")
end


local function get_log_config()
    local log_max_size, size_unit
    local ok1, msg_dir = utils.get_data(const.data_key.logger.dir)
    if not ok1 then
        if pl_string.startswith(msg_dir, "err_key_does_not_exist") then
            ok1 = true
            msg_dir = ""
        end
    end

    local ok2, msg_size = utils.get_data(const.data_key.logger.max_size)
    local ok3, msg_level = utils.get_data(const.data_key.logger.level)
    if (ok1 and ok2 and ok3) then
        if (msg_dir and msg_size and msg_level) then
            log_max_size = tonumber(msg_size)
            if (log_max_size >= 1024) then
                log_max_size = log_max_size / 1024
                size_unit= "GB"
            else
                size_unit = "MB"
            end

            if ("WARNING" == msg_level) then
                msg_level = "WARN"
            end

            return msg_level, log_max_size, size_unit, msg_dir
        else
            return nil, nil, nil, nil
        end
    end

    utils.log_error("get log config failed")
    utils.error_exit(500)
end


local request_method = ngx.var.request_method

if (not utils.check_permission("LogManage")) then
    utils.error_exit(403)
end

if ("GET" == request_method) then
    local log_level, log_max_size, size_unit, log_root_path = get_log_config()
    if (not (log_level and log_max_size and size_unit and log_root_path)) then
        -- set default logger configuration
        set_log_config("DEBUG", 512, "MB", "/var/tmp/iiot_aliyun/logger")
        log_level, log_max_size, size_unit, log_root_path = get_log_config()
    end

    if (not check_config_params(log_level, log_max_size, size_unit, log_root_path)) then
        utils.log_error("internal error: invalid log config data" .. log_level .. tostring(log_max_size) .. size_unit .. log_root_path)
        utils.error_exit(400, "log config invalid")
    end

    local res = {}
    res["LogLevel"] = log_level
    res["LogMaxSize"] = log_max_size
    res["SizeUnit"] = size_unit
    res["LogRootPath"] = log_root_path
    log_config_raw = json.encode(res)
    utils.error_exit(200, log_config_raw)

elseif ("POST" == request_method) then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end

    log_level = args["LogLevel"]
    log_max_size = tonumber(args["LogMaxSize"]) 
    size_unit = args["SizeUnit"]
    log_root_path = args["LogRootPath"]
    if (not check_config_params(log_level, log_max_size, size_unit, log_root_path)) then
        utils.log_warn("invalid paramert: log_level=%s, log_max_size=%s, size_unit=%s, log_root_path=%s", 
                log_level,
                log_max_size,
                size_unit,
                log_root_path)
        utils.error_exit(400, "log config invalid")
    end

    --if (not pl_path.exists(log_root_path)) then
    --    ok, err = pl_dir.makepath(log_root_path)
    --    if nil == ok then
    --        utils.error_exit(500, "create new path failed" .. err)
    --    end
    --end

    set_log_config(log_level, log_max_size, size_unit, log_root_path)
    utils.error_exit(200)
end

utils.error_exit(400, "unsupported request method")
------ EOF ------
