local const = require("const")
local json = require("json")
local utils = require("utils")
local hmac = require "resty.hmac"

local auth_check = {}

local function urlEncode(s)  
     s = string.gsub(s, "([^%w%.%- ])", function(c) return string.format("%%%02X", string.byte(c)) end)  
    return string.gsub(s, " ", "+")  
end  

local function get_secret_from_ssc(user)
    local ok, msg = utils.get_data(const.data_key.eweb.user)
    
    if ok and msg then
        local user_list = json.decode(msg)
    
        if nil ~= user_list and nil ~= user_list["UserList"] and nil ~= user_list["UserList"][user] then
            return true, user_list["UserList"]["Password"]
        end
    end

    if user == const.default_user.name then
        return true, const.default_user.password
    end
   
    return false
end

local function is_param_legal(args)
 
    if (nil == args) then
        return false, "args must not be null", "iot.common.InvalidParams"
    end
    
    local format = args["Format"]
    local version = args["Version"]
    local access_key_id = args["AccessKeyId"]
    local signature = args["Signature"]
    local signature_method = args["SignatureMethod"]
    local time_stamp = args["Timestamp"]
    local signature_version = args["SignatureVersion" ]
    local signature_nonce = args["SignatureNonce"]

    if access_key_id == nil or type(access_key_id) ~= "string" then
        return false, "AccessKeyId is invalid", "iot.common.InvalidParams"
    end

    if signature == nil or type(signature) ~= "string" then
        return false, "signature is not exist", "iot.common.InvalidParams"
    end

    if signature_method == nil or type(signature_method) ~= "string" or signature_method ~= "HMAC-SHA1" then
        return false, "sigature method is invalid", "iot.common.InvalidParams"
    end

    --FIXME: 时间要在15分钟内.
    if time_stamp == nil or type(time_stamp) ~= "string" then
        return false, "timestamp is invalid", "iot.common.InvalidParams"
    end
   
    --FIXME: 唯一值必须在session有效期内是唯一的
    if signature_nonce == nil or type(signature_nonce) ~= "string" then
        return false, "signature none is invalid", "iot.common.InvalidParams"
    end

    --FIXME: 版本号确定
    if version == nil or type(version) ~= "string" then
        return false, "version is invalid", "iot.common.InvalidParams"
    end

    return true, "ok"
end

function auth_check.is_public_request_params_valid(args)
    local public_args = {}
    local sign = ""

    local ret_code, ret_msg, ret_id = is_param_legal(args)
    if ret_code == false then
        return ret_code, ret_msg, ret_id
    end
    
    ret_code,  secret = get_secret_from_ssc(args["AccessKeyId"])
    if ret_code == false then
        return false, "access key id is not exist.", "iot.openapi."
    end
    -- group the arguments
    for key, value in pairs(args) do
        if key ~= "Signature" then
            public_args[key] = value
        else
            sign = value
        end
    end

    -- sort & encode public arguments
    local tkeys = {}
    for k in pairs(public_args) do table.insert(tkeys, k) end
    table.sort(tkeys)
    local args_sorted_encoded = ""
    for _, k in ipairs(tkeys) do
        local key_encoded = urlEncode(k)
        local val_encoded = urlEncode(public_args[k])

        string.gsub(key_encoded, '+', '%20')
        string.gsub(key_encoded, '*', '%2A')
        string.gsub(key_encoded, "%%7E", '~')
        
        string.gsub(val_encoded, '+', '%20')
        string.gsub(val_encoded, '*', '%2A')
        string.gsub(val_encoded, "%%7E", '~')
        
        args_sorted_encoded = args_sorted_encoded..key_encoded.."="..val_encoded.."&"
    end
    
    local args_sorted = string.sub(args_sorted_encoded, 1, #args_sorted_encoded - 1)
      
    -- 构造签名字符串 
    local string_to_sign = 'GET'..'&'..urlEncode("/")..'&'..urlEncode(args_sorted)

    local hmac_sha1 = hmac:new(secret.."&", hmac.ALGOS.SHA1)
    if not hmac_sha1 then
        return false, "failed to create the hmac_sha1 object", "iot.system.SystemException"
    end

    local ok = hmac_sha1:update(string_to_sign)
    if not ok then
        return false, "failed to calc signature", "iot.system.SystemException"
    end  

    local mac = hmac_sha1:final()
    local final_sign = ngx.encode_base64(mac)
    
    if final_sign == sign then
        return true, "ok"
    else
        return false, "signature is not correct", "iot.common.InvalidSignature"
    end
end

return auth_check

